-- **************************************************************************
-- * TitanReagentTracker.lua
-- *
-- * By: L'ombra
-- **************************************************************************

-- ******************************** Constants *******************************
TITAN_REAGENTTRACKER_ID = "ReagentTracker"
TITAN_REAGENTTRACKER_VERSION = "1.0"

-- ******************************** Variables *******************************
-- English class name
local class = select(2, UnitClass("player"))

-- database of buff spells and reagents
local buffs = {
	["DEATHKNIGHT"] = {
		{
			-- Raise Dead
			["reagentIds"] = {
				37201,
			},
			["spellIds"] = {
				48289,
			},
		},
	},
	["DRUID"] = {
		{
			-- Gift of the Wild
			["reagentIds"] = {
				-- if multiple reagents are listed, reagentIds[i] is the reagent for spellIds[i]
				[1] = 17021,
				[2] = 17026,
				[3] = 22148,
				[4] = 44605,
			},
			["spellIds"] = {
				-- indexed by rank if different reagent for each rank
				[1] = 21849,
				[2] = 21850,
				[3] = 26991,
				[4] = 48470,
			},
		},
		{
			-- Rebirth
			["reagentIds"] = {
				[1] = 17034,
				[2] = 17035,
				[3] = 17036,
				[4] = 17037,
				[5] = 17038,
				[6] = 22147,
				[7] = 44614,
			},
			["spellIds"] = {
				[1] = 20484,
				[2] = 20739,
				[3] = 20742,
				[4] = 20747,
				[5] = 20748,
				[6] = 26994,
				[7] = 48477,
			},
		},
	},
	["MAGE"] = {
		{
			-- Arcane Brilliance
			["reagentIds"] = {
				-- if only one reagent is listed, it's the reagent for all spellIds
				17020,
			},
			["spellIds"] = {
				-- no particular order if different spells/ranks uses the same reagent
				23028,
				27127,
				43002,
			},
		},
		{
			-- Teleport
			["reagentIds"] = {
				17031,
			},
			["spellIds"] = {
				32272,
				32271,
				3567,
				3566,
				3565,
				3563,
				3562,
				3561,
				33690,
				35715,
				53140,
			},
		},
		{
			-- Portal
			["reagentIds"] = {
				17032,
			},
			["spellIds"] = {
				32267,
				32266,
				11420,
				11419,
				11418,
				11417,
				11416,
				10059,
				33691,
				35717,
				53142,
			},
		},
		{
			-- Slow fall
			["reagentIds"] = {
				17056,
			},
			["spellIds"] = {
				130,
			},
		},
	},
	["PALADIN"] = {
		{
			-- Greater Blessing
			["reagentIds"] = {
				21177,
			},
			["spellIds"] = {
				25894,
				25782,
				25918,
				25916,
				27143,
				27141,
				48937,
				48933,
				48938,
				48934,
			},
		},
		{
			-- Divine Intervention
			["reagentIds"] = {
				17033,
			},
			["spellIds"] = {
				19752,
			},
		},
	},
	["PRIEST"] = {
		{
			-- Prayer of Fortitude
			["reagentIds"] = {
				[1] = 17028,
				[2] = 17029,
				[3] = 17029,
				[4] = 44615,
			},
			["spellIds"] = {
				[1] = 21562,
				[2] = 21564,
				[3] = 25392,
				[4] = 48162,
			},
		},
		{
			-- Levitate
			["reagentIds"] = {
				17056,
			},
			["spellIds"] = {
				1706,
			},
		}
	},
	["SHAMAN"] = {
		{
			-- Reincarnation
			["reagentIds"] = {
				17030,
			},
			["spellIds"] = {
				20608,
			},
		},
		{
			-- Water breathing
			["reagentIds"] = {
				17057,
			},
			["spellIds"] = {
				131,
			},
		},
		{
			-- Water walking
			["reagentIds"] = {
				17058,
			},
			["spellIds"] = {
				546,
			},
		},
	},
	["WARLOCK"] = {
		{
			-- Soul Shard
			["reagentIds"] = {
				6265,
			},
			["spellIds"] = {
				697,
				6201,
			},
		},
		{
			-- Ritual of Doom
			["reagentIds"] = {
				16583,
			},
			["spellIds"] = {
				18540,
			},
		},
	}
}

-- store spells that are known here later
local possessed = {}

-- ******************************** Functions *******************************

function TitanPanelReagentTrackerButton_OnLoad(self)
	self.registry = {
		id = TITAN_REAGENTTRACKER_ID,
		version = TITAN_REAGENTTRACKER_VERSION,
		menuText = "Reagent Tracker",
		tooltipTitle = "Reagent Tracker Info", 
		tooltipTextFunction = "TitanPanelReagentTrackerButton_GetTooltipText",
		savedVariables = {
			TrackReagent1 = false,
			TrackReagent2 = false,
			TrackReagent3 = false,
			TrackReagent4 = false,
			ShowSpellIcons = false,
		}
	}
	
	-- don't continue if no buffs are associated with the class
	if buffs[class] then
		self:RegisterEvent("PLAYER_ENTERING_WORLD")
		self:RegisterEvent("LEARNED_SPELL_IN_TAB")
		self:RegisterEvent("BAG_UPDATE")
	end
end


function TitanPanelReagentTrackerButton_OnEvent(self, event, ...)
	if event ~= "BAG_UPDATE" then
		for index, buff in ipairs(buffs[class]) do
			for i = #buff.spellIds, 1, -1 do
				if GetSpellInfo(GetSpellInfo(buff.spellIds[i])) then
					-- add this spell to our "known spells" table
					possessed[index] = {
						["reagentId"] = buff.reagentIds[i] or buff.reagentIds[1],
						["spellId"] = buff.spellIds[i],
					}
					break
				end
			end
		end
	end
	
	TitanPanelReagentTrackerButton_UpdateButton()

	TitanPanelButton_UpdateTooltip(self)
end


function TitanPanelReagentTrackerButton_UpdateButton()
	local tracking = false
	local totalWidth = 0
	local offset = 0
	local name = "TitanPanelReagentTrackerButtonReagent"

	for index, buff in ipairs(possessed) do
		local button = _G[name..index]
		local nextButton = _G[name..(index + 1)]
		local nextAnchor = "LEFT"
		local nextOffset = 0
		
		-- show/hide reagent trackers
		if TitanGetVar(TITAN_REAGENTTRACKER_ID, "TrackReagent"..index) then
			local icon = _G[name..index.."Icon"]
			button:Show()
			-- display spell or reagent icon
			if TitanGetVar(TITAN_REAGENTTRACKER_ID, "ShowSpellIcons") then
				icon:SetTexture(select(3, GetSpellInfo(buff.spellId)))
			else
				icon:SetTexture(GetItemIcon(buff.reagentId))
			end
			
			-- current number of reagents
			button:SetText(GetItemCount(buff.reagentId))
			
			if nextButton then
				nextAnchor = "RIGHT"
				nextOffset = 6
			end
			
			button:SetWidth(icon:GetWidth() + button:GetTextWidth())
			totalWidth = totalWidth + button:GetWidth()
			
			offset = offset + 1
			tracking = true
		else
			button:Hide()
		end
		
		-- fix offset to next reagent tracker
		if nextButton then
			nextButton:SetPoint("LEFT", button, nextAnchor, nextOffset, 0)
		end
	end
	
	-- show label if no tracking is enabled
	local none = _G[name.."None"]
	if tracking then
		none:Hide()
	else
		none:Show()
		totalWidth = none:GetWidth() + 8
	end
	
	-- adjust width so other plugins are properly offset
	TitanPanelReagentTrackerButton:SetWidth(totalWidth + ((offset - 1) * 8))
end


function TitanPanelReagentTrackerButton_GetTooltipText()
	local tooltipText = " "
	
	-- reagent info in tooltip
	for index, buff in ipairs(possessed) do
		if TitanGetVar(TITAN_REAGENTTRACKER_ID, "TrackReagent"..index) then
			tooltipText = tooltipText.."\n"..GetItemInfo(buff.reagentId).."\t"..TitanUtils_GetHighlightText(GetItemCount(buff.reagentId))
		end
	end
	
	if #tooltipText > 1 then
		return tooltipText
	else
		return " \nNo reagents tracked."
	end
end


function TitanPanelRightClickMenu_PrepareReagentTrackerMenu()
	TitanPanelRightClickMenu_AddTitle(TitanPlugins[TITAN_REAGENTTRACKER_ID].menuText)
	
	-- add meny entry for each possessed spell
	for index, buff in ipairs(possessed) do
		local info = {}
		info.text = "Track "..GetItemInfo(buff.reagentId)
		info.value = "TrackReagent"..index
		info.func = TitanPanelReagentTracker_ToggleVar
		info.checked = TitanGetVar(TITAN_REAGENTTRACKER_ID, "TrackReagent"..index)
		info.keepShownOnClick = 1
		UIDropDownMenu_AddButton(info)
	end
	
	TitanPanelRightClickMenu_AddSpacer()
	
	local info = {}
	info.text = "Show spell icons"
	info.value = "ShowSpellIcons"
	info.func = TitanPanelReagentTracker_ToggleVar
	info.checked = TitanGetVar(TITAN_REAGENTTRACKER_ID, "ShowSpellIcons")
	info.keepShownOnClick = 1
	UIDropDownMenu_AddButton(info)
end


function TitanPanelReagentTracker_ToggleVar(self)
	TitanToggleVar(TITAN_REAGENTTRACKER_ID, self.value)
	
	TitanPanelReagentTrackerButton_UpdateButton()
end